# -*- coding: utf-8 -*-
# Copyright 2017 Gooee.com, LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "LICENSE" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from .compat import json


class Resource(object):
    """Objectify a Response."""

    def __init__(self, response):

        try:
            self.json = response.json()
        except json.decoder.JSONDecodeError:
            # This happens on a DELETE or a response that returns
            # nothing. We could improve this to only raise when it isn't
            # that case... but we'd need to know it was expected... or
            # not. This would also occur when DJANGO is in DEBUG mode
            # and the traceback webpage is returned in the response.
            self.json = None
        self.text = response.text
        self.elapsed = response.elapsed
        self.headers = response.headers
        self.reason = response.reason
        self.status_code = response.status_code
        self.request = response.request

    def __repr__(self):
        return '<{} {} {}:{}>'.format(
            self.request.method,
            self.request.url,
            self.status_code,
            self.reason
        )



# -*- coding: utf-8 -*-
# Copyright 2017 Gooee.com, LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "LICENSE" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import functools

import requests

from .exceptions import InternetConnectionError
from .models import Resource


def resource(func):
    """Converts the response to a Resource."""

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        try:
            response = func(*args, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise InternetConnectionError(e)

        return Resource(response)

    return wrapper



# -*- coding: utf-8 -*-
# Copyright 2017 Gooee.com, LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "LICENSE" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from __future__ import unicode_literals

import logging

__author__ = 'Gooee LLC'
__email__ = 'dairon@gooee.com'
__version__ = '0.0.1'

from .client import GooeeClient  # noqa


def set_stream_logging(level=logging.DEBUG, format_string=None):
    """
    Add a stdout log handler with provided level to the SDK operations.
        >>> import logging
        >>> import gooee
        >>> gooee.set_stream_logging(logging.INFO)
    :type level: int
    :param level: Logging level, e.g. ``logging.INFO``
    :type format_string: str
    :param format_string: Log message format
    """
    if format_string is None:
        format_string = '%(asctime)s %(name)s [%(levelname)s] %(message)s'

    logger = logging.getLogger('gooee')
    logger.setLevel(level)
    handler = logging.StreamHandler()
    handler.setLevel(level)
    formatter = logging.Formatter(format_string)
    handler.setFormatter(formatter)
    logger.addHandler(handler)


class NullHandler(logging.Handler):
    """
    Set up logging to ``/dev/null`` like a library is supposed to.
    http://docs.python.org/3.3/howto/logging.html#configuring-logging-for-a-library
    """

    def emit(self, record):
        pass


logging.getLogger('gooee').addHandler(NullHandler())



# -*- coding: utf-8 -*-
# Copyright 2017 Gooee.com, LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "LICENSE" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from os import environ

from six import string_types
from six.moves import urllib_parse

from .exceptions import InvalidResourcePath

GOOEE_API_URL = environ.get('GOOEE_API_URL', 'https://dev-api.gooee.io/')
GOOEE_API_PATH = urllib_parse.urlparse(GOOEE_API_URL).path


def format_path(path, api_base_url=GOOEE_API_URL):
    error_msg = 'The path argument must be a string that begins with "/"'
    if not isinstance(path, string_types):
        raise InvalidResourcePath(error_msg)

    # Using the HTTP shortcut
    if path.startswith('/'):
        return urllib_parse.urljoin(api_base_url, path.lstrip('/'))

    raise InvalidResourcePath(error_msg)



# -*- coding: utf-8 -*-
# Copyright 2017 Gooee.com, LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "LICENSE" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from __future__ import unicode_literals

from platform import platform

import requests
from six import string_types

from .compat import json
from .decorators import resource
from .exceptions import IllegalHttpMethod
from . import __version__
from .utils import (
    format_path,
    GOOEE_API_URL
)


class GooeeClient(object):
    """Gooee HTTP client class."""

    allowed_methods = ('get', 'post', 'put', 'patch', 'delete', 'options')

    def __init__(self, api_base_url=GOOEE_API_URL):
        self.api_base_url = api_base_url
        self.auth_token = ''

    def _request(self, method, path, headers=None, data=None, params=None):
        """Request helper."""
        if method not in self.allowed_methods:
            msg = 'HTTP method {} not supported. Needs to be one of: {}'.format(
                method, self.allowed_methods)
            raise IllegalHttpMethod(msg)

        url = format_path(path, self.api_base_url)

        headers_final = self.default_headers
        headers_final.update(headers or {})
        if not headers_final['Authorization']:
            headers_final.pop('Authorization')

        if data and not isinstance(data, string_types):
            data = json.dumps(data)

        response = getattr(requests, method)(
            url, headers=headers_final, data=data, params=params)

        return response

    def authenticate(self, username, password):
        """Authenticate to get the JWT token and stash it for use with future
        requests.
        """
        payload = {
            'username': username,
            'password': password,
        }
        token = self.post('/auth/login', data=payload).json['token']
        self.auth_token = 'JWT {token}'.format(token=token)

    @property
    def default_headers(self):
        """Default headers to talk to the API with."""
        headers = {
            'Content-Type': 'application/json',
            'Authorization': self.auth_token,
            'User-Agent': 'gooee-python-sdk {version} ({system})'.format(
                version=__version__,
                system=platform(),
            )
        }
        return headers

    @resource
    def get(self, path, params=None):
        return self._request('get', path, params=params)

    @resource
    def post(self, path, data=None, params=None):
        return self._request('post', path, data=data, params=params)

    @resource
    def put(self, path, data=None, params=None):
        return self._request('put', path, data=data, params=params)

    @resource
    def patch(self, path, data=None, params=None):
        return self._request('patch', path, data=data, params=params)

    @resource
    def delete(self, path, data=None, params=None):
        return self._request('delete', data=data, params=params)

    @resource
    def options(self, path, params=None):
        return self._request('options', path, params=params)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

__version__ = '0.0.1'

if sys.argv[-1] == 'tag':
    os.system("git tag -a %s -m 'version %s'" % (__version__, __version__))
    os.system("git push --tags")
    sys.exit()

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()

if sys.argv[-1] == 'test':
    os.system('py.test')
    sys.exit()

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

requirements = ["requests>=2.0", ]

# Add Python 2.6-specific dependencies
if sys.version_info[:2] < (2, 7):
    requirements.append('simplejson')

setup(
    name='gooee',
    version=__version__,
    description='Official Gooee SDK for Python',
    long_description=readme + '\n\n' + history,
    author='Gooee LLC',
    author_email='dairon@gooee.com',
    url='https://github.com/GooeeIOT/gooee-python-sdk',
    packages=[
        'gooee',
    ],
    package_dir={'gooee':
                 'gooee'},
    include_package_data=True,
    install_requires=requirements,
    license="Apache",
    zip_safe=False,
    keywords='gooee, IoT, lighting',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content'
    ],
    test_suite='tests'
)



# -*- coding: utf-8 -*-
# Copyright 2017 Gooee.com, LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "LICENSE" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from requests.exceptions import ConnectionError


class GooeeException(Exception):
    """Base class for all the Gooee errors."""
    pass


class IllegalHttpMethod(GooeeException):
    pass


class InvalidResourcePath(GooeeException):
    pass


class UnknownEndpoint(GooeeException):
    pass


class UnsupportedEndpoint(GooeeException):
    pass


class InternetConnectionError(ConnectionError):
    """
    Wraps requests.exceptions.ConnectionError in order to provide a more
    intuitively named exception.
    """
    pass
